turf/minigame/F_STM_Memory	// all minigames should be of this type
	icon = 'F_STM_Memory_Color.dmi'
	icon_state = "BASE"
	name = "F_STM_Memory"
	var
		atom/movable/minigame_piece/F_STM_Memory_Color/Color
		atom/movable/minigame_piece/F_STM_Memory_Direction/Direction
		atom/movable/minigame_piece/F_STM_Memory_Base/Red/Red
		atom/movable/minigame_piece/F_STM_Memory_Base/Blue/Blue
		atom/movable/minigame_piece/F_STM_Memory_Base/Green/Green
		atom/movable/minigame_piece/F_STM_Memory_Base/Yellow/Yellow
		mob/North
		mob/South
		mob/East
		mob/West
		started = FALSE
		active = TRUE
		turnOrder = list(SOUTH, WEST, NORTH, EAST)
		currentPlayers[0]
		currentTurn
		colorList[0]
		speed = 5
		currentAttemptPlace = 1
		lastCorrect
		highName
		highScore
	verb
		HighScore()
			set src in view(1)
			set desc = "See the high score for this game."
			if(highName)
				usr << "<font color = blue><b>[highName]</b></font> holds the record on this game with a pattern length of [highScore]."
			else
				usr << "There is not yet a high score for this game."



	proc
		CurrentPlayerQuit(M)
			if(M == North)
				North = null
			if(M == South)
				South = null
			if(M == East)
				East = null
			if(M == West)
				West = null
			currentPlayers -= M
			oview(src, 1) << "\proper[M] has quit the game."
			if(currentPlayers.len)
				oview(src, 1) << "The current turn is being passed to the next player."
				AdvanceTurn()
			else
				oview(src, 1) << "No more players in the current round."
				Winner()

		PlayerQuit(M)
			oview(src, 1) << "\proper[M] has quit the game."
			if(M == North)
				North = null
			if(M == South)
				South = null
			if(M == East)
				East = null
			if(M == West)
				West = null
			currentPlayers -= M

		Reset()
			started = FALSE
			active = TRUE
			currentAttemptPlace = 1
			colorList.Cut()
			ClearTurn()
			SetCurrentPlayers()
			lastCorrect = ""

		ClearTurn()
			Direction.Update(null)

		Flash(Dir)
			Color.Update(Dir)
			sleep(4)
			Color.Update(null)

		ParseAttempt(mob/M, Dir)
			if(!(Dir in list(NORTH, SOUTH, EAST, WEST))) return
			if(!active && MyTurn(M) && Dir == colorList[currentAttemptPlace])
				Flash(Dir)
				if(currentAttemptPlace == colorList.len)
					sleep(5)
					lastCorrect = copytext(M.name,1)
					AddColor()
					NextTurn()
				else
					currentAttemptPlace += 1
			else if(!active && MyTurn(M))
				oview(src,1) << "[M] blew it and has been removed from the round."
				ohearers(src,1) << 'buzzerheavy.wav'
				currentPlayers -= M
				if(!currentPlayers.len)
					oview(src,1) << "Game over!"
					Winner()
				else
					NextTurn()

		Winner()
			if(lastCorrect)
				oview(src,1) << "\proper[lastCorrect] wins with a pattern length of [colorList.len -1]."
				if(colorList.len-1 > highScore)
					oview(src, 1) << "<font color=red><b>New high score!"
					highScore = colorList.len-1
					highName = lastCorrect
			else
				oview(src,1) << "No winner."
			Reset()

		Pause()
			active = TRUE

		NextTurn()
			Pause()
			AdvanceTurn()
			currentAttemptPlace = 1
			PlayColors()
			WaitForAttempt()

		WaitForAttempt()
			active = FALSE

		PlayColors()
			for(var/V in colorList)
				Color.Update(V)
				sleep(speed)
//				Color.Update(null)
//				sleep(3)
			Color.Update(null)

		AddColor()
			colorList += pick(NORTH, SOUTH, EAST, WEST)

		PlayerExists(turn)
			switch(turn)
				if(NORTH)
					if(North && North in currentPlayers) return TRUE
				if(SOUTH)
					if(South && South in currentPlayers) return TRUE
				if(WEST)
					if(West && West in currentPlayers) return TRUE
				if(EAST)
					if(East && East in currentPlayers) return TRUE
			return FALSE

		AdvanceTurn()
			var/found = FALSE
			var/tries = 1
			while(!found)
				currentTurn += 1
				tries += 1
				if(currentTurn>=5) currentTurn = 1
				if(PlayerExists(turnOrder[currentTurn]))
					found = TRUE
					ShowTurn()



		ShowTurn()
			Direction.Update(TextDir(turnOrder[currentTurn]))

		MyTurn(mob/M)
			var/mob/CT
			switch(turnOrder[currentTurn])
				if(SOUTH)
					CT = South
				if(EAST)
					CT = East
				if(WEST)
					CT = West
				if(NORTH)
					CT = North
			if(M == CT) return TRUE
			return FALSE

		SetCurrentPlayers()
			currentPlayers.Cut()
			for(var/mob/M in list(North, South, East, West))
				if(M)
					currentPlayers += M

		StartGame()
			if(!started)
				SetCurrentPlayers()
				started = TRUE
				active = FALSE
				currentTurn = SetFirstTurn()
				oview(src, 1) << "Game starting in two seconds."
				sleep(20)
				AdvanceTurn()
				AddColor()
				PlayColors()

		SetFirstTurn()
			var/list/L = new()
			if(North)
				L += 3
			if(South)
				L += 1
			if(West)
				L += 2
			if(East)
				L += 4
			var/P = pick(L)
			return P

	New()
		..()
		Color = new(src)
		Direction = new(src)
		Red = new(src)
		Blue = new(src)
		Yellow = new(src)
		Green = new(src)

	instructions()
		// override to provide game instructions.
		set src in view(1)
		set desc = "Read the game instructions."
		var/HTML = {"
<center><h2>Memory Instructions</h2></center>
<table align=center width=80%><tr><td>
Memory is a simple game of pattern memorization and regurgitation.
On each players turn, the game will flash the current color pattern,
and the player must replay it.  After each successful turn, the pattern
will be increased by one color.  If you make a mistake, you are removed
from the current round, and play continues without you.  The round
ends when the last player makes a mistake.  The winner is the last to
have successfully repeated a pattern.
<br><br>
To join a game, simply bump into the game board on an unoccupied side.
You will start playing at the beginning of the next round.  Press the
center key to begin a new game.
<br><br>
Patterns are repeated using the arrow keys.  Up = Red, Right = Green,
Down = Blue and Left = Yellow.  You may also click the appropriate color
on the game board itself.
</td></tr></table>
"}
		usr << browse(HTML)

	GameBumped(mob/M)
		// M just bumped into this game
		switch(M.dir)
			if(NORTH)
				if(!South)
					switch(alert(M, "Would you like to join this game?", "Join game?", "Yes", "No"))
						if("No") return
					oview(src,1) << "[M] has joined the game as the south player."
					South = M
					M.client.Game = src
			if(SOUTH)
				if(!North)
					switch(alert(M, "Would you like to join this game?", "Join game?", "Yes", "No"))
						if("No") return
					oview(src,1) << "[M] has joined the game as the north player."
					North = M
					M.client.Game = src
			if(EAST)
				if(!West)
					switch(alert(M, "Would you like to join this game?", "Join game?", "Yes", "No"))
						if("No") return
					oview(src,1) << "[M] has joined the game as the east player."
					West = M
					M.client.Game = src
			if(WEST)
				if(!East)
					switch(alert(M, "Would you like to join this game?", "Join game?", "Yes", "No"))
						if("No") return
					oview(src,1) << "[M] has joined the game as the west player."
					East = M
					M.client.Game = src
		if(started)
			M << "You will start playing when the next round begins."

	PlayerInput(mob/M, Dir)
		/* mob just input Dir into this game.
			Dir 0 is the center numpad key. */
		if(Dir == 0 && !started)

			StartGame()
			return
		ParseAttempt(M, Dir)


	Quit(mob/M)
		// Called when M quit the game by using the esc key.
		if(MyTurn(M))
			CurrentPlayerQuit(M)
		else
			PlayerQuit(M)
		M.client.Game = null

	proc
		TextDir(Dir)
			switch(Dir)
				if(NORTH)
					return "NORTH"
				if(SOUTH)
					return "SOUTH"
				if(EAST)
					return "EAST"
				if(WEST)
					return "WEST"


atom/movable/minigame_piece
	F_STM_Memory_Color
		layer = OBJ_LAYER+1
		name = "F_STM_Memory"
		icon = 'F_STM_Memory_Color.dmi'
		proc
			Update(Dir)
				switch(Dir)
					if(NORTH)
						flick("RED", src)
						ohearers(src.loc, 1) << 'ping1.wav'

					if(SOUTH)
						//icon_state = "BLUE"
						flick("BLUE", src)
						ohearers(src.loc, 1) << 'ping2.wav'

					if(EAST)
						//icon_state = "GREEN"
						flick("GREEN", src)
						ohearers(src.loc, 1) << 'ping3.wav'

					if(WEST)
						//icon_state = "YELLOW"
						flick("YELLOW", src)
						ohearers(src.loc, 1) << 'ping4.wav'
					else
						icon_state = ""

	F_STM_Memory_Direction
		name = "F_STM_Memory"
		icon = 'F_STM_Memory_Color.dmi'
		proc
			Update(Dir)
				icon_state = Dir

	F_STM_Memory_Base
		icon = 'F_STM_Memory_Color.dmi'
		layer = OBJ_LAYER
		var
			turf/minigame/F_STM_Memory/Owner
		Red
			icon_state="BRED"
			dir = NORTH
		Blue
			icon_state= "BBLUE"
			dir = SOUTH
		Yellow
			icon_state = "BYELLOW"
			dir = WEST
		Green
			icon_state = "BGREEN"
			dir = EAST
		New()
			..()
			Owner=loc
		Click()
			Owner.PlayerInput(usr, src.dir)
			..()

client
	Del()
		if(Game)
			Game.Quit(mob)
			Game = null
		..()